package org.kozyrev.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.Socket;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.BoundedRangeModel;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.ToolTipManager;
import javax.swing.border.BevelBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TreeModelListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;

import org.apache.log4j.Logger;
import org.kozyrev.gui.barchart.BarChart;
import org.kozyrev.gui.barchart.DefaultBarChartModel;
import org.kozyrev.gui.barchart.BarChartModel.Bar;
import org.kozyrev.gui.classchooser.ClassControlBox;
import org.kozyrev.gui.classchooser.ClassList;
import org.kozyrev.gui.classchooser.ClassListCellRenderer;
import org.kozyrev.gui.classchooser.ClassListModel;
import org.kozyrev.gui.classchooser.Option;
import org.kozyrev.gui.util.ClassFilter;
import org.kozyrev.gui.util.ClassFilter.FilterChangeListener;
import org.kozyrev.log.LogConfigurator;
import org.kozyrev.log.LogConfigurator.LoggerConfiguration;
import org.kozyrev.model.ClassRecord;
import org.kozyrev.model.FrameRecord;
import org.kozyrev.model.MethodRecord;
import org.kozyrev.model.ProfileModel;
import org.kozyrev.model.ThreadRecord;
import org.kozyrev.model.xml.DomXMLParser;
import org.kozyrev.net.ConnectionController;

/**
 * @author 
 * 
 */
public class Main extends JFrame {

    private static final long serialVersionUID = 8760253928179097082L;

    private DefaultBarChartModel myAllocChartModel = new DefaultBarChartModel();

    private final JFileChooser myPropOpener = new JFileChooser();

    private final JFileChooser mySaver = new JFileChooser();

    private ProfileModel myProfileModel;

    private final JTree myTree = new JTree();

    private final JPanel myTabs = new JPanel();

    private final JPanel myStatusBar = new JPanel();

    private final JLabel myStatusText = new JLabel();

    private final JPanel myFilters = new JPanel();

    private BarChart myBarChart;

    private final JTable myMethodTable = new JTable();

    private MethodsTableModel myTableModel;

    private ClassList myClassList;

    private final ClassListModel myClassListModel = ClassListModel.getInstance();

    private ClassControlBox myFilterControlBox;

    private final ClassFilter myClassFilter = ClassFilter.getInstance();

    private static final Logger LOG = Logger.getLogger(Main.class);

    private final Action myOpenAction = new OpenAction();

    private final Action mySaveAction = new SaveAsAction();

    private final Action myExecuteAction = new ExecuteAction();

    private final Action myHelpAboutAction = new HelpAboutAction();

    private final Action myConfigureAction = new ConfigureAction();

    private final Action myStopAction = new StopProfileAction();

    private final Action myShowFiltersAction = new ShowFiltersAction();

    private final Action myExitAction = new ExitAction();

    private Socket mySocket;

    private final MainStateManager myStateManager = new MainStateManager();

    private int myUserSelectedPort;

    private int myFreePort;

    private class OpenAction extends AbstractAction {

        /**
         * 
         */
        private static final long serialVersionUID = 1077762990750165623L;

        public OpenAction() {
            ClassLoader cl = this.getClass().getClassLoader();
                        
            ImageIcon openIcon = new ImageIcon(cl
                    .getResource("images/document-open.png"));

            this.putValue(Action.SMALL_ICON, openIcon);
            this.putValue(Action.NAME, "Open...");
            this.putValue(Action.SHORT_DESCRIPTION, "Open saved profiling");
            this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(
                    KeyEvent.VK_O, ActionEvent.CTRL_MASK));
        }

        public void actionPerformed(ActionEvent e) {

            if (!(myPropOpener.showOpenDialog(Main.this) == JFileChooser.APPROVE_OPTION)) {
                return;
            }

            loadModelFromFile(myPropOpener.getSelectedFile());

            myStateManager.setState(MainStateManager.STATE_AFTER_PROFILING);
        }
    }

    private class SaveAsAction extends AbstractAction {

        /**
         * 
         */
        private static final long serialVersionUID = 1077762990750165623L;

        public SaveAsAction() {
            ClassLoader cl = this.getClass().getClassLoader();
            ImageIcon saveIcon = new ImageIcon(cl
                    .getResource("images/document-save-as.png"));

            this.putValue(Action.SMALL_ICON, saveIcon);
            this.putValue(Action.NAME, "Save as...");
            this.putValue(Action.SHORT_DESCRIPTION, "Save profiling results");
            this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(
                    KeyEvent.VK_S, ActionEvent.CTRL_MASK));
        }

        public void actionPerformed(ActionEvent e) {

            if ((mySaver.showSaveDialog(Main.this) == JFileChooser.APPROVE_OPTION)) {
                String path = mySaver.getSelectedFile().getAbsolutePath();
                if (!path.endsWith(".profite")) {
                    path += ".profile";
                }

                copyFile("output/profile.profile.xml", path);
            }
        }

    }

    private class ExecuteAction extends AbstractAction {

        private ProfileWizard w = new ProfileWizard();

        /**
         * 
         */
        private static final long serialVersionUID = -6328429456739897691L;

        public ExecuteAction() {
            ClassLoader cl = this.getClass().getClassLoader();
            ImageIcon execIcon = new ImageIcon(cl
                    .getResource("images/utilities-system-monitor.png"));

            this.putValue(Action.SMALL_ICON, execIcon);
            this.putValue(Action.NAME, "Execute...");
            this.putValue(Action.SHORT_DESCRIPTION, "Start profiling");
            this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(
                    KeyEvent.VK_F6, 0));
        }

        public void actionPerformed(ActionEvent e) {

            if (w.showRunDialog() == ProfileWizard.RUN_OPTION) {
                try {
                    String[] properties = ProfileConfig.readProperties();
                    myUserSelectedPort = Integer.parseInt(properties[3]);

                    myFreePort = ConnectionController
                            .getFreePort(myUserSelectedPort);

                    if (myFreePort != myUserSelectedPort) {
                        JOptionPane.showMessageDialog(Main.this, "Port "
                                + myUserSelectedPort + " is busy. " + myFreePort
                                + " was selected!", "Warning!",
                                JOptionPane.WARNING_MESSAGE);

                        properties[3] = "" + myFreePort;
                        ProfileConfig.storeProperties(properties);
                    }

                    Runtime.getRuntime().exec(
                            "java -jar -javaagent:util/profile.jar -Dprofile.properties="
                                    + "util/profile.properties " + " "
                                    + w.getVMArguments() + " " + "\""
                                    + w.getSelectedFile() + "\"" + " "
                                    + w.getProgramArguments());

                    Waiter waiterThread = new Waiter();
                    waiterThread.start();

                    myTabs.setVisible(false);
                    myStateManager.setState(MainStateManager.STATE_PROFILING);
                } catch (IOException e1) {
                    e1.printStackTrace();
                }

            }
        }

    }

    private class ConfigureAction extends AbstractAction {

        /**
         * 
         */
        private static final long serialVersionUID = 824778789646240123L;

        public ConfigureAction() {
            ClassLoader cl = this.getClass().getClassLoader();
            ImageIcon cfgIcon = new ImageIcon(cl
                    .getResource("images/preferences-system.png"));

            this.putValue(Action.SMALL_ICON, cfgIcon);
            this.putValue(Action.NAME, "Configure...");
            this.putValue(Action.SHORT_DESCRIPTION, "Configure profiler");
            this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(
                    KeyEvent.VK_F9, 0));
        }

        public void actionPerformed(ActionEvent e) {
            (new ProfileConfig(Main.this, "Profiling configuration", true))
                    .setVisible(true);

        }

    }

    private class HelpAboutAction extends AbstractAction {

        /**
         * 
         */
        private static final long serialVersionUID = 6505888737724819323L;

        private ImageIcon myAboutIconSmall;

        private ImageIcon myAboutIconBig;

        public HelpAboutAction() {
            ClassLoader cl = this.getClass().getClassLoader();
            myAboutIconSmall = new ImageIcon(cl
                    .getResource("images/help-browser-small.png"));
            myAboutIconBig = new ImageIcon(cl
                    .getResource("images/dialog-information.png"));

            this.putValue(Action.SMALL_ICON, myAboutIconSmall);
            this.putValue(Action.NAME, "About...");
            this.putValue(Action.SHORT_DESCRIPTION, "About program");
        }

        public void actionPerformed(ActionEvent e) {
            JOptionPane.showMessageDialog(Main.this, aboutMessage(), "About",
                    JOptionPane.INFORMATION_MESSAGE, myAboutIconBig);
        }

        private String aboutMessage() {
            return "<html> <H1>Visual-JIP</H1>\n"
                    +

                    "Simple profiling tool using bytecode instrumentation for java applications"
                    + "\nBased on 'The Java Interactive Profiler'"
                    +

                    "\n\n<html>This product includes software and data developed by:"
                    + "\n<html>Tango Desktop Project"
                    + "<html>&nbsp;-&nbsp;<a href=0>http://tango.freedesktop.org/"
                    + "\n<html>JIP  The Java Interactive Profiler"
                    + "<html>&nbsp;-&nbsp;<a href=0>jiprof.sourceforge.net</a>"
                    + "\n<html>ASM - Java bytecode manipulation framework"
                    + "<html>&nbsp;-&nbsp;<a href=0>http://asm.objectweb.org/"
                    + "\n<html>Log4j Project"
                    + "<html>&nbsp;-&nbsp;<a href=0>http://logging.apache.org/log4j/"
                    + "\n<html>JUnit testing framework"
                    + "<html>&nbsp;-&nbsp;<a href=0>http://www.junit.org/";
        }
    }

    private class StopProfileAction extends AbstractAction {

        private static final long serialVersionUID = 3092049787212594355L;

        private final ImageIcon myStopImage;

        public StopProfileAction() {
            ClassLoader cl = this.getClass().getClassLoader();
            myStopImage = new ImageIcon(cl.getResource("images/process-stop.png"));

            this.putValue(Action.SMALL_ICON, myStopImage);
            this.putValue(Action.NAME, "Stop");
            this.putValue(Action.SHORT_DESCRIPTION, "Stop profiling");
            this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(
                    KeyEvent.VK_F12, 0));
        }

        public void actionPerformed(ActionEvent e) {
            if (mySocket != null) {
                OutputStream os;
                try {
                    os = mySocket.getOutputStream();
                    BufferedOutputStream out = new BufferedOutputStream(os);
                    out.write("stop".getBytes());
                    out.write('\r');
                    out.flush();
                } catch (IOException e1) {
                    LOG.error(e1.getMessage());
                    JOptionPane
                            .showMessageDialog(
                                    Main.this,
                                    "Unable to perform action"
                                            + "\nPlease, see manual or contact support.",
                                    "Error", JOptionPane.ERROR_MESSAGE);

                    myStateManager.setState(MainStateManager.STATE_BEFORE_PROFILING);
                }

            }
        }
    }

    private class ShowFiltersAction extends AbstractAction {
        /**
         * 
         */
        private static final long serialVersionUID = -1029224346334854579L;

        private final ImageIcon myFilterImage;

        public ShowFiltersAction() {
            ClassLoader cl = this.getClass().getClassLoader();
            myFilterImage = new ImageIcon(cl.getResource("images/edit-find.png"));

            this.putValue(Action.SMALL_ICON, myFilterImage);
            this.putValue(Action.NAME, "Show/Hide filters");
            this.putValue(Action.SHORT_DESCRIPTION, "Show/Hide class filters");
            this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(
                    KeyEvent.VK_F3, 0));
        }

        public void actionPerformed(ActionEvent e) {
            myFilters.setVisible(!myFilters.isVisible());
        }
    }

    private class ExitAction extends AbstractAction {

        private static final long serialVersionUID = -3267354861368260870L;

        private final ImageIcon myExitImage;

        public ExitAction() {
            ClassLoader cl = this.getClass().getClassLoader();
            myExitImage = new ImageIcon(cl
                    .getResource("images/system-log-out.png"));

            this.putValue(Action.SMALL_ICON, myExitImage);
            this.putValue(Action.NAME, "Exit");
            this.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(
                    KeyEvent.VK_Q, ActionEvent.ALT_MASK));
        }

        public void actionPerformed(ActionEvent e) {
            System.exit(0);
        }
    }

    private class StackTreeCellRenderer extends DefaultTreeCellRenderer {

        /**
         * 
         */
        private static final long serialVersionUID = -519050882153926264L;

        @Override
        public Component getTreeCellRendererComponent(JTree tree, Object value,
                boolean sel, boolean expanded, boolean leaf, int row,
                boolean hasFocus) {

            if (value instanceof List) {
                super.getTreeCellRendererComponent(tree, "Application "/*
                                                                         * +
                                                                         * jarOpener.getSelectedFile().getName()
                                                                         */, sel, expanded, leaf, row, hasFocus);

                setToolTipText(null);
                setIcon(null);
                myStatusText.setText("");
                return this;
            } else {
                if (value instanceof ThreadRecord) {
                    super.getTreeCellRendererComponent(tree, "Thread id "
                            + ((ThreadRecord) value).getID(), sel, expanded,
                            leaf, row, hasFocus);

                    setToolTipText(null);
                    setIcon(null);
                    myStatusText.setText("");

                    return this;
                } else {
                    if (sel) {
                        double time = ((double) ((FrameRecord) value).getTime()) / 1000000;
                        myStatusText.setText("Total time: " + time + " ms");
                    }

                    String fullName = ((FrameRecord) value).getMethod()
                            .getName();
                    String shortName = fullName
                            .substring(fullName.indexOf(":") + 1);

                    int calls = ((FrameRecord) value).getCallCount();

                    fullName = fullName.replace("<", "&lt;");
                    fullName = fullName.replace(">", "&gt;");

                    if ("<init>".equals(shortName)) {
                        shortName = ((FrameRecord) value).getMethod()
                                .getDeclaringClass().getShortName()
                                + " constructor";
                    }

                    String info = "<html><b>"
                            + shortName
                            + "</b><br>"
                            + "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"
                            + fullName
                            + "<br>"
                            + "<i>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;call count:</i> "
                            + calls;

                    super.getTreeCellRendererComponent(tree, info, sel,
                            expanded, leaf, row, hasFocus);

                    setIcon(null);
                    return this;
                }
            }
        }
    }

    private class StackTreeModel implements TreeModel {

        private ArrayList<TreeModelListener> myModelListeners = new ArrayList<TreeModelListener>();

        private List<ThreadRecord> myRoot;

        public StackTreeModel(List<ThreadRecord> root) {
            myRoot = root;
        }

        public void addTreeModelListener(TreeModelListener l) {
            myModelListeners.add(l);
        }

        @SuppressWarnings("unchecked")
        public Object getChild(Object parent, int index) {
            if (parent instanceof List) {
                List<ThreadRecord> list = ((List<ThreadRecord>) parent);
                return list.get(index);
            } else {
                if (parent instanceof ThreadRecord) {

                    List<FrameRecord> frames = ((ThreadRecord) parent)
                            .getFrames();
                    int current = -1;
                    for (FrameRecord frame : frames) {
                        if (myClassFilter.accept(frame.getMethod()
                                .getDeclaringClass().getName())) {
                            current++;
                        }

                        if (current == index) {
                            return frame;
                        }
                    }

                    return ((ThreadRecord) parent).getFrames().get(index);
                } else {
                    if (parent instanceof FrameRecord) {

                        List<FrameRecord> frames = ((FrameRecord) parent)
                                .getCalledFrames();
                        int current = -1;
                        for (FrameRecord frame : frames) {
                            if (myClassFilter.accept(frame.getMethod()
                                    .getDeclaringClass().getName())) {
                                current++;
                            }

                            if (current == index) {
                                return frame;
                            }
                        }

                        return ((FrameRecord) parent).getCalledFrames().get(
                                index);
                    }
                }

            }

            return null;
        }

        @SuppressWarnings("unchecked")
        public int getChildCount(Object parent) {
            if (parent instanceof List) {
                List<ThreadRecord> list = ((List<ThreadRecord>) parent);
                return list.size();
            } else {
                if (parent instanceof ThreadRecord) {

                    List<FrameRecord> frames = ((ThreadRecord) parent)
                            .getFrames();
                    int count = 0;
                    for (FrameRecord frame : frames) {
                        if (myClassFilter.accept(frame.getMethod()
                                .getDeclaringClass().getName())) {
                            ++count;
                        }
                    }

                    return count;
                } else {
                    if (parent instanceof FrameRecord) {
                        List<FrameRecord> frames = ((FrameRecord) parent)
                                .getCalledFrames();
                        int count = 0;
                        for (FrameRecord frame : frames) {
                            if (myClassFilter.accept(frame.getMethod()
                                    .getDeclaringClass().getName())) {
                                ++count;
                            }
                        }

                        return count;
                    }
                }

            }

            return 0;
        }

        @SuppressWarnings("unchecked")
        public int getIndexOfChild(Object parent, Object child) {
            if (parent instanceof List) {
                List<ThreadRecord> list = ((List<ThreadRecord>) parent);
                return list.indexOf(child);
            } else {
                if (parent instanceof ThreadRecord) {
                    return ((ThreadRecord) parent).getFrames().indexOf(child);
                } else {
                    if (parent instanceof FrameRecord) {
                        return ((FrameRecord) parent).getCalledFrames()
                                .indexOf(child);
                    }
                }

            }
            return 0;
        }

        public Object getRoot() {
            return myRoot;
        }

        public boolean isLeaf(Object node) {
            return (node instanceof FrameRecord)
                    && (!((FrameRecord) node).hasChildren());
        }

        public void removeTreeModelListener(TreeModelListener l) {
            for (int i = 0; i < myModelListeners.size(); i++) {
                if (myModelListeners.get(i) == l) {
                    myModelListeners.remove(i);
                    return;
                }
            }
        }

        public void valueForPathChanged(TreePath path, Object newValue) {
            LOG.debug("*** valueForPathChanged : " + path + " --> " + newValue);
        }

    }

    private class MethodsTableModel extends AbstractTableModel {

        /**
         * 
         */
        private static final long serialVersionUID = -2153607699151984975L;

        private final ArrayList<TableRow> myData = new ArrayList<TableRow>();

        private final String[] myCols = new String[] { "Method name", "Class name",
                "Package", "Time (nanos)", "Call count" };

        public void addRow(TableRow t) {
            myData.add(t);
        }

        public void clear() {
            myData.clear();
        }

        public int getColumnCount() {
            return 5;
        }

        public int getRowCount() {
            return myData.size();
        }

        public Object getValueAt(int rowIndex, int columnIndex) {
            switch (columnIndex) {
            case (0): {
                String method = myData.get(rowIndex).myMethod.getShortName();
                if ("<init>".equals(method)) {
                    method = myData.get(rowIndex).myMethod.getDeclaringClass()
                            .getShortName()
                            + " constructor";
                }

                return method;
            }
            case (1): {
                return myData.get(rowIndex).myMethod.getDeclaringClass()
                        .getShortName();
            }
            case (2): {
                return myData.get(rowIndex).myMethod.getDeclaringClass()
                        .getPackage();
            }
            case (3): {
                return myData.get(rowIndex).myMethod.getTotalTime();
            }
            case (4): {
                return myData.get(rowIndex).myCount;
            }
            }
            ;

            return null;
        }

        @Override
        public String getColumnName(int column) {
            return myCols[column];
        }

        public void sort(Comparator<TableRow> cmp) {
            Collections.sort(myData, cmp);
        }
    }

    private class TableSelectionListener implements ListSelectionListener {

        long myCounter = 0;

        private ListSelectionModel mySelectionModel;

        public void valueChanged(ListSelectionEvent e) {
            if (!e.getValueIsAdjusting()) {
                mySelectionModel = (ListSelectionModel) e.getSource();

                myCounter = 0;
                int[] selection = getSelectedIndices(mySelectionModel
                        .getMinSelectionIndex(), mySelectionModel
                        .getMaxSelectionIndex());
                if (selection.length == 0) {
                    myStatusText.setText("");
                } else {
                    for (int i = 0; i < selection.length - 1; i++) {
                        myCounter += Long.parseLong(myTableModel.getValueAt(
                                selection[i], 3).toString());
                    }

                    myCounter += Long.parseLong(myTableModel.getValueAt(
                            selection[selection.length - 1], 3).toString());
                }

            }

            myStatusText.setText("Total time: " + (double) myCounter / 1000000
                    + " ms");
        }

        protected int[] getSelectedIndices(int start, int stop) {
            if ((start == -1) || (stop == -1)) {
                return new int[0];
            }

            int guesses[] = new int[stop - start + 1];
            int index = 0;
            for (int i = start; i <= stop; i++) {
                if (mySelectionModel.isSelectedIndex(i)) {
                    guesses[index++] = i;
                }
            }

            int realthing[] = new int[index];
            System.arraycopy(guesses, 0, realthing, 0, index);
            return realthing;
        }

    }

    private class MyTableHeaderListener extends MouseAdapter {
        private int myLastColumn = -1;

        @Override
        public void mouseClicked(MouseEvent e) {
            if (e.getSource() instanceof JTableHeader) {
                JTableHeader header = (JTableHeader) e.getSource();
                int column = header.columnAtPoint(e.getPoint());
                column = header.getColumnModel().getColumn(column)
                        .getModelIndex();

                int sign = 0;
                if (myLastColumn == column) {
                    sign = -1;
                    myLastColumn = -1;
                } else {
                    sign = 1;
                    myLastColumn = column;
                }

                switch (column) {
                case (0): {
                    myTableModel.sort(getMethodComparator(sign));
                    break;
                }
                case (1): {
                    myTableModel.sort(getClassComparator(sign));
                    break;
                }
                case (2): {
                    myTableModel.sort(getPackageComparator(sign));
                    break;
                }
                case (3): {
                    myTableModel.sort(getTimeComparator(sign));
                    break;
                }
                case (4): {
                    myTableModel.sort(getCallCountComparator(sign));
                    break;
                }
                }
                ;

                header.getTable().repaint();
            }

        }

        public Comparator<TableRow> getMethodComparator(final int sign) {
            return new Comparator<TableRow>() {
                public int compare(TableRow o1, TableRow o2) {
                    String method1 = o1.getMyMethod().getShortName();
                    if ("<init>".equals(method1)) {
                        method1 = o1.getMyMethod().getDeclaringClass()
                                .getShortName()
                                + " constructor";
                    }
                    String method2 = o2.getMyMethod().getShortName();
                    if ("<init>".equals(method2)) {
                        method2 = o2.getMyMethod().getDeclaringClass()
                                .getShortName()
                                + " constructor";
                    }
                    return sign * method1.compareTo(method2);
                }
            };
        }

        public Comparator<TableRow> getClassComparator(final int sign) {
            return new Comparator<TableRow>() {
                public int compare(TableRow o1, TableRow o2) {
                    return sign
                            * o1.getMyMethod().getDeclaringClass()
                                    .getShortName().compareTo(
                                            o2.getMyMethod()
                                                    .getDeclaringClass()
                                                    .getShortName());
                }
            };
        }

        public Comparator<TableRow> getPackageComparator(final int sign) {
            return new Comparator<TableRow>() {
                public int compare(TableRow o1, TableRow o2) {
                    return sign
                            * o1.getMyMethod().getDeclaringClass().getPackage()
                                    .compareTo(
                                            o2.getMyMethod()
                                                    .getDeclaringClass()
                                                    .getPackage());
                }
            };
        }

        public Comparator<TableRow> getTimeComparator(final int sign) {
            return new Comparator<TableRow>() {
                public int compare(TableRow o1, TableRow o2) {
                    return sign
                            * Long.valueOf(o1.getMyMethod().getTotalTime())
                                    .compareTo(
                                            Long.valueOf(o2.getMyMethod()
                                                    .getTotalTime()));
                }
            };
        }

        public Comparator<TableRow> getCallCountComparator(final int sign) {
            return new Comparator<TableRow>() {
                public int compare(TableRow o1, TableRow o2) {
                    return sign
                            * Long.valueOf(o1.getMyCount()).compareTo(
                                    Long.valueOf(o2.getMyCount()));
                }
            };
        }
    }

    private class TableRow {
        private int myCount;

        private MethodRecord myMethod;

        public TableRow(MethodRecord method, int count) {
            myCount = count;
            myMethod = method;
        }

        public int getMyCount() {
            return myCount;
        }

        public MethodRecord getMyMethod() {
            return myMethod;
        }

    }

    class Waiter extends Thread {

        public void run() {

            String title = getTitle();
            setTitle(title + " - profiling");

            try {
                mySocket = null;
                try {
                    mySocket = ConnectionController.getSocket(myFreePort);
                } catch (IOException e1) {
                    LOG.error("     ", e1);

                    JOptionPane
                            .showMessageDialog(
                                    Main.this,
                                    "Unable to perform action: connect to profiler"
                                    + "\nPlease, see manual or contact support.",
                                    "Error", JOptionPane.ERROR_MESSAGE);

                    myStateManager.setState(MainStateManager.STATE_BEFORE_PROFILING);
                    return;
                }

                BufferedReader in;
                try {
                    in = new BufferedReader(new InputStreamReader(mySocket
                            .getInputStream()));
                    String cmd = in.readLine();

                    if ("finish".equals(cmd)) {
                        loadModelFromFile(new File("output/profile.profile.xml"));
                        myStateManager.setState(MainStateManager.STATE_AFTER_PROFILING);
                        myTabs.setVisible(true);
                    } else {
                        JOptionPane
                                .showMessageDialog(
                                        Main.this,
                                        "Profiling has failed. Results unavailable"
                                                + "\nPlease, see manual or contact support",
                                        "", JOptionPane.ERROR_MESSAGE);

                        LOG
                                .error("Profiling has failed. Results unavailable"
                                        + "\nPlease, see manual or contact support");

                        myStateManager.setState(MainStateManager.STATE_BEFORE_PROFILING);
                    }
                } catch (IOException e) {
                    JOptionPane
                            .showMessageDialog(
                                    Main.this,
                                    "Profiling has failed. Results unavailable"
                                            + "\nPlease, see manual or contact support",
                                    "", JOptionPane.ERROR_MESSAGE);

                    LOG.error("Connection error", e);
                    myStateManager.setState(MainStateManager.STATE_BEFORE_PROFILING);
                }
    
            } finally {
                setTitle(title);
                String[] properties = ProfileConfig.readProperties();
                properties[3] = "" + myUserSelectedPort;
                ProfileConfig.storeProperties(properties);
            }           

        }

    }

    private class PackageFilterListener implements FilterChangeListener {
        public void filterChanded(ClassFilter source) {
            createBarChartModel();
            createTableModel();
            createStackTree();
        }
    }

    private class PackageListDataListener implements ListDataListener {

        public void contentsChanged(ListDataEvent e) {
            createPackageFilter();
        }

        public void intervalAdded(ListDataEvent e) {
            createPackageFilter();

        }

        public void intervalRemoved(ListDataEvent e) {
            createPackageFilter();

        }

    }

    public static void main(String[] args) {

        try {
            LogConfigurator.configure(LoggerConfiguration.DEFAULT);
            new Main();

        } finally {

        }
    }

    public Main() {
        super("Visual-JIP");

        loadClassFilters();

        FileFilter profileFilter;

        profileFilter = new FileFilter() {

            @Override
            public boolean accept(File f) {
                return (f.isDirectory())
                        || f.getAbsolutePath().endsWith(".profile");
            }

            @Override
            public String getDescription() {
                return "Program profile [*.profile]";
            }

        };

        mySaver.setFileFilter(profileFilter);
        mySaver.setFileSelectionMode(JFileChooser.FILES_ONLY);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                saveClassFilters();
            }
        });

        myPropOpener.setApproveButtonText("Open");
        myPropOpener.setFileSelectionMode(JFileChooser.FILES_ONLY);
        myPropOpener.setFileFilter(profileFilter);

        createGUI();

        setDefaultCloseOperation(EXIT_ON_CLOSE);
        setSize(900, 700);
        setLocation();
        setVisible(true);
    }

    private void createGUI() {
        Container c = getContentPane();

        JTabbedPane tabbedPane = new JTabbedPane(JTabbedPane.BOTTOM,
                JTabbedPane.SCROLL_TAB_LAYOUT);

        myTabs.setLayout(new BorderLayout());
        myTabs.add(tabbedPane, BorderLayout.CENTER);

        tabbedPane.addTab("Call stack", getStackFrameTab());
        tabbedPane.addTab("Methods", getMethodListTab());
        tabbedPane.addTab("Object allocations", getObjAllocTab());

        JToolBar tools = new JToolBar();
        tools.add(myExecuteAction);
        tools.add(myStopAction);
        tools.add(Box.createHorizontalStrut(5));
        tools.add(myConfigureAction);
        tools.add(Box.createHorizontalStrut(5));
        
        tools.add(myOpenAction);
        tools.add(mySaveAction);
        
        tools.add(Box.createHorizontalGlue());
        tools.add(new JToggleButton(myShowFiltersAction));
        //tools.add(Box.createHorizontalStrut(5));
        
        myClassList = ClassList.getInstance();
        myClassList.setModel(myClassListModel);
        myClassList.setCellRenderer(new ClassListCellRenderer());
        myClassList
                .addMouseListener(new ClassListCellRenderer.PackageListElementListener());

        myClassList
                .addKeyListener(new ClassListCellRenderer.ClassListKeyListener());
        myClassList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        myFilterControlBox = new ClassControlBox();
        myFilterControlBox.setList(myClassList);
        myFilterControlBox.setPreferredSize(new Dimension(250, 80));

        myClassListModel.addListDataListener(new PackageListDataListener());

        myTabs.setVisible(false);

        setJMenuBar(createMenu());

        // classes
        myFilters.setPreferredSize(new Dimension(250, 78));
        myFilters.setLayout(new BorderLayout());

        JList classes = ClassList.getInstance();
        classes.setCellRenderer(new ClassListCellRenderer());

        JScrollPane pane;
        myFilters.add(pane = new JScrollPane(classes));
        pane
                .setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

        myFilters.add(myFilterControlBox, BorderLayout.SOUTH);
        myFilters.setVisible(false);

        JLabel header = new JLabel("Please, select filters");
        header.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

        myFilters.add(header, BorderLayout.NORTH);

        myTabs.add(myFilters, BorderLayout.EAST);

        myStatusBar.setPreferredSize(new Dimension(200, 24));
        myStatusBar.setLayout(new BorderLayout());
        myStatusText.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
        myStatusBar.add(myStatusText, BorderLayout.WEST);
        myStatusBar.setBorder(BorderFactory
                .createBevelBorder(BevelBorder.LOWERED));
        c.add(myStatusBar, BorderLayout.SOUTH);

        c.add(myTabs);
        c.add(tools, BorderLayout.NORTH);

        // associate states with components

        MainStateManager.STATE_BEFORE_PROFILING.addAction(myOpenAction);
        MainStateManager.STATE_BEFORE_PROFILING.addAction(myExecuteAction);
        MainStateManager.STATE_BEFORE_PROFILING.addAction(myConfigureAction);
        MainStateManager.STATE_BEFORE_PROFILING.addAction(myHelpAboutAction);

        MainStateManager.STATE_PROFILING.addAction(myStopAction);
        MainStateManager.STATE_PROFILING.addAction(myHelpAboutAction);

        MainStateManager.STATE_AFTER_PROFILING.addAction(myOpenAction);
        MainStateManager.STATE_AFTER_PROFILING.addAction(mySaveAction);
        MainStateManager.STATE_AFTER_PROFILING.addAction(myExecuteAction);
        MainStateManager.STATE_AFTER_PROFILING.addAction(myConfigureAction);
        MainStateManager.STATE_AFTER_PROFILING.addAction(myHelpAboutAction);
        MainStateManager.STATE_AFTER_PROFILING.addAction(myShowFiltersAction);

        myStateManager.setState(MainStateManager.STATE_BEFORE_PROFILING);
    }

    private JMenuBar createMenu() {
        JMenuBar menuBar = new JMenuBar();

        JMenu file = new JMenu("File");
        JMenu profile = new JMenu("Profile");
        JMenu help = new JMenu("Help");

        file.add(myOpenAction);
        file.add(mySaveAction);
        file.addSeparator();
        file.add(myExitAction);

        profile.add(myExecuteAction);
        profile.add(myStopAction);
        profile.addSeparator();
        profile.add(myConfigureAction);

        help.add(myHelpAboutAction);

        menuBar.add(file);
        menuBar.add(profile);
        menuBar.add(help);

        return menuBar;
    }

    private JPanel getStackFrameTab() {
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));

        panel.add(new JScrollPane(myTree));
        myTree.setModel(new DefaultTreeModel(null));
        myTree.setCellRenderer(new StackTreeCellRenderer());

        // Enable tool tips
        ToolTipManager.sharedInstance().registerComponent(myTree);

        return panel;
    }

    private JPanel getObjAllocTab() {
        JPanel panel = new JPanel();

        panel.setLayout(new BorderLayout());
        panel.add(myBarChart = new BarChart(myAllocChartModel));
        JScrollBar scrollBar = new JScrollBar();
        scrollBar.setModel(myBarChart.getBoundedRangeModel());
        panel.add(scrollBar, BorderLayout.EAST);
        scrollBar.getModel().addChangeListener(new ChangeListener() {
            public void stateChanged(ChangeEvent e) {
                myBarChart.repaint();
            }
        });

        myBarChart.addMouseWheelListener(new MouseWheelListener() {
            public void mouseWheelMoved(MouseWheelEvent e) {
                int rotation = e.getWheelRotation();
                BoundedRangeModel rangeModel = myBarChart.getBoundedRangeModel();
                if (rotation + rangeModel.getValue() < 0) {
                    rangeModel.setValue(0);
                } else {
                    if (rotation + rangeModel.getValue() > rangeModel
                            .getMaximum()) {
                        rangeModel.setValue(rangeModel.getMaximum());
                    } else {
                        rangeModel.setValue(rotation + rangeModel.getValue());
                    }
                }
            }
        });

        return panel;
    }

    private JPanel getMethodListTab() {
        JPanel panel = new JPanel();
        panel.setLayout(new BoxLayout(panel, BoxLayout.X_AXIS));

        panel.add(new JScrollPane(myMethodTable));
        myMethodTable.getTableHeader().addMouseListener(
                new MyTableHeaderListener());

        myMethodTable.getSelectionModel().addListSelectionListener(
                new TableSelectionListener());

        return panel;
    }

    private void loadClassFilters() {
        Properties props = new Properties();
        String propsFile = "filters.properties";

        FileInputStream is = null;

        try {
            is = new FileInputStream(propsFile);
            props.load(is);
        } catch (FileNotFoundException e1) {
            /*ignoring*/
        } catch (IOException e) {
            System.err.print("Unable to open ");
            System.err.print(propsFile);
            
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {/* Ignoring */
                }
            }
        }

        String packages = props.getProperty("count", "0");
        int count = Integer.parseInt(packages);

        for (int i = 0; i < count; i++) {
            String pkg = props.getProperty("package" + (String.valueOf(i + 1)),
                    "null");
            myClassListModel.addElement(new Option(pkg, false));
        }

        createPackageFilter();
    }

    /**
     * 
     */
    private void saveClassFilters() {
        Properties props = new Properties();
        String propsFile = "filters.properties";

        props.setProperty("count", String.valueOf(myClassListModel.size()));

        for (int i = 0; i < myClassListModel.size(); i++) {
            props.setProperty("package" + (String.valueOf(i + 1)),
                    ((Option) myClassListModel.get(i)).getText());
        }

        FileOutputStream os = null;
        try {
            os = new FileOutputStream(propsFile);
            props.store(os, "no comments");
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (os != null) {
                try {
                    os.close();
                } catch (IOException e) {/* Ignoring */
                }
            }
        }
    }

    @SuppressWarnings("unused")
    private void copyFile(String from, String to) {
        try {
            FileInputStream fileFrom = new FileInputStream(from);
            FileOutputStream fileTo = new FileOutputStream(to);

            BufferedInputStream bufferFrom = new BufferedInputStream(fileFrom);
            BufferedOutputStream bufferTo = new BufferedOutputStream(fileTo);

            while (bufferFrom.available() > 0) {
                bufferTo.write(bufferFrom.read());
            }

            bufferFrom.close();
            bufferTo.close();

        } catch (FileNotFoundException e) {
            System.err.println(e.getMessage());
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void loadModelFromFile(File file) {

        myTabs.setVisible(true);

        myProfileModel = DomXMLParser.parseFile(file);

        createStackTree();
        createTableModel();
        createBarChartModel();

        myClassFilter.addFilterChangeListener(new PackageFilterListener());
    }

    private void createStackTree() {
        myTree.setModel(new StackTreeModel(myProfileModel.getThreads()));
    }

    /**
     * creates package filter by packageListModel
     */
    @SuppressWarnings("unchecked")
    private void createPackageFilter() {
        myClassFilter.clear();

        Enumeration<Option> en = (Enumeration<Option>) myClassListModel
                .elements();
        while (en.hasMoreElements()) {
            Option currentOption = en.nextElement();

            if (currentOption.isAllowed()) {
                myClassFilter.addPackageRegExp(currentOption.getText());
            }
        }
    }

    private void createBarChartModel() {
        myAllocChartModel.clear();

        List<ClassRecord> ClasList;

        ClasList = myProfileModel.getClasses();

        long maxWeight = 0;
        for (ClassRecord rec : ClasList) {
            if (maxWeight < rec.getAllocationCount()) {
                maxWeight = rec.getAllocationCount();
            }
        }

        for (ClassRecord rec : ClasList) {

            if (myClassFilter.accept(rec.getName())) {

                if (rec.getAllocationCount() < 0.3f * maxWeight) {
                    myAllocChartModel.addBar(rec.getShortName(), rec
                            .getAllocationCount(), Color.BLUE);
                } else if (rec.getAllocationCount() < 0.6f * maxWeight) {
                    myAllocChartModel.addBar(rec.getShortName(), rec
                            .getAllocationCount(), Color.ORANGE);
                } else {
                    myAllocChartModel.addBar(rec.getShortName(), rec
                            .getAllocationCount(), Color.PINK);
                }

            }
            ;

        }

        myAllocChartModel.setSorter(new Comparator<Bar>() {
            public int compare(Bar o1, Bar o2) {
                return Integer.valueOf(o2.getWeight()).compareTo(
                        Integer.valueOf(o1.getWeight()));
            }
        });
    }

    private void createTableModel() {
        Set<MethodRecord> methodSet;

        HashMap<MethodRecord, Integer> methods = new HashMap<MethodRecord, Integer>();

        for (ThreadRecord tr : (myProfileModel.getThreads())) {
            for (FrameRecord fr : tr.getFrames()) {
                traverseFrame(fr, methods);
            }
        }

        methodSet = methods.keySet();
        myTableModel = new MethodsTableModel();

        for (Iterator<MethodRecord> it = methodSet.iterator(); it.hasNext();) {

            MethodRecord rec = it.next();

            if (myClassFilter.accept(rec.getDeclaringClass().getName())) {
                myTableModel.addRow(new TableRow(rec, methods.get(rec)));
            }
        }

        myMethodTable.setModel(myTableModel);

    }

    private void traverseFrame(FrameRecord frame, Map<MethodRecord, Integer> map) {
        if (map.containsKey(frame.getMethod())) {
            map.put(frame.getMethod(), map.get(frame.getMethod())
                    + frame.getCallCount());
        } else {
            map.put(frame.getMethod(), frame.getCallCount());
        }

        for (FrameRecord f : frame.getCalledFrames()) {
            traverseFrame(f, map);
        }
    }
    
    private void setLocation() {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int x = (int) ((screenSize.getWidth() - getWidth()) / 2);
        int y = (screenSize.height - getHeight()) / 2;
        setLocation(new Point(x, y));
    }

}
