package org.kozyrev.gui;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.SpringLayout;
import javax.swing.border.BevelBorder;
import javax.swing.table.AbstractTableModel;

public class ProfileConfig extends JDialog {
    
    /**
     * 
     */
    private static final long serialVersionUID = 2747742193645738005L;
    static final String DEFAULT_PROFILE = "on";
    static final String DEFAULT_REMOVE = "off";
    static final String DEFAULT_PORT = "15599";
    static final String DEFAULT_MAX_THREAD_DEPTH = "-1";
    static final String DEFAULT_THREAD_COMPACT_THRESHOLD = "10";
    static final String DEFAULT_MAX_METHOD_COUNT = "-1";
    static final String DEFAULT_METHOD_COMPACT_THRESHOLD = "10";
    static final String DEFAULT_OBJECT_ALLOC = "off";
    
    static final String DEFAULT_PROPERTY_FILE_NAME = "util/profile.properties";
    
    private String[] myProperties = new String[8];
    
    private String[] myPropNames = new String[] {"profile", "remote", "track object allocation",
            "port", "thread depth", "thread compact treshold", "max method count", "method compact treshold"};

    private final Action cancelAction = new CancelAction();
    
    private final Action saveAction = new SaveAction();
    
    private class BooleanProps extends AbstractTableModel {

        /**
         * 
         */
        private static final long serialVersionUID = -80666769904567976L;

        public String getColumnName(int col) {
            return null;
        }
        
        public int getRowCount() { 
            return 3;
        }
        
        public int getColumnCount() {
            return 2;
        }
        
        @SuppressWarnings("unchecked")
        public Class getColumnClass(int c) {
            return (c == 1) ? Boolean.class
                            : String.class;
        }
        
        public Object getValueAt(int row, int col) {
            if (col == 0) {
                return myPropNames[row];
            } else {
                return (new Boolean(myProperties[row].equals("on")));
            }
        }
        
        public boolean isCellEditable(int row, int col) {
            return col == 1;
        }
        
        public void setValueAt(Object value, int row, int col) {
            
            if ((Boolean) value) {
                myProperties[row] = "on";
            } else {
                myProperties[row] = "off";
            }
            
            fireTableCellUpdated(row, col);
        }
        
    }
    
    private class SaveAction extends AbstractAction {
        /**
         * 
         */
        private static final long serialVersionUID = -982281892948390334L;

        public SaveAction() {

            this.putValue(Action.NAME, "Save");
        }
        
        public void actionPerformed(ActionEvent e) {
            storeProperties(myProperties);
            dispose();
        }  
    }
    
    private class CancelAction extends AbstractAction {
        /**
         * 
         */
        private static final long serialVersionUID = -982281892948390334L;

        public CancelAction() {
            this.putValue(Action.NAME, "Close");
        }
        
        public void actionPerformed(ActionEvent e) {
            dispose();
        }  
    }
    
    private class IntProps extends AbstractTableModel {

        /**
         * 
         */
        private static final long serialVersionUID = -6932306398396417235L;

        public String getColumnName(int col) {
            return null;
        }
        
        public int getRowCount() { 
            return 5;
        }
        
        public int getColumnCount() {
            return 2;
        }
        
        @SuppressWarnings("unchecked")
        public Class getColumnClass(int c) {
            return (c == 1) ? Integer.class
                            : String.class;
        }
        
        public Object getValueAt(int row, int col) {
            if (col == 0) {
                return myPropNames[row + 3];
            } else {
                return Integer.parseInt(myProperties[row + 3]);
            }

        }
        
        public boolean isCellEditable(int row, int col) {
            return col == 1;
        }
        
        public void setValueAt(Object value, int row, int col) {
            myProperties[row + 3] = value.toString();            
            
            fireTableCellUpdated(row, col);
        }
        
    }
    
    public ProfileConfig(Frame owner, String title, boolean modal) {
        super(owner, title, modal);
        setResizable(false);
        setSize(new Dimension(200, 200));
        
        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                ProfileConfig.this.dispose();
            }           
            
        });
        
        myProperties = readProperties();
        createGUI();
             
        pack();
        setLocation();
    }
    
    public static String[] readProperties() {

        Properties props = new Properties();
        String propsFile = DEFAULT_PROPERTY_FILE_NAME;
        
        FileInputStream is = null;
        
        try {
            File f = new File(propsFile);
            if (!f.exists()) {
                f.createNewFile();
            }
            is = new FileInputStream(propsFile);
            props.load(is);
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();
        } catch (IOException e) {
            System.err.print("Unable to open ");
            System.err.print(propsFile);
            e.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {/*Ignoring*/}
            }
        }
        
      
        String[] properties = new String[8];
        /*profiler*/ properties[0] = props.getProperty("profiler", DEFAULT_PROFILE);
        /*remote*/ properties[1] = props.getProperty( "remote", DEFAULT_REMOVE);
        /*objectAlloc*/properties[2] = props.getProperty("track.object.alloc", DEFAULT_OBJECT_ALLOC);
        /*port*/ properties[3]= props.getProperty("port", DEFAULT_PORT);
        /*threadDepth*/ properties[4] = props.getProperty("thread-depth", DEFAULT_MAX_THREAD_DEPTH);
        /*threadCompactThreshold*/ properties[5]= props.getProperty("thread.compact.threshold.ms", DEFAULT_THREAD_COMPACT_THRESHOLD);
        /*maxMethodCount*/properties[6] = props.getProperty("max-method-count", DEFAULT_MAX_METHOD_COUNT);
        /*methodCompactThreshold*/ properties[7]= props.getProperty("method.compact.threshold.ms", DEFAULT_METHOD_COMPACT_THRESHOLD);
        
        return properties;        
    }
    
    public static void storeProperties(String[] properties) {
        Properties props = new Properties();
        String propsFile = DEFAULT_PROPERTY_FILE_NAME;
        
        FileInputStream is = null;
        
        try {
            is = new FileInputStream(propsFile);
            props.load(is);
        } catch (FileNotFoundException e1) {
            e1.printStackTrace();
        } catch (IOException e) {
            System.err.print("Unable to open ");
            System.err.print(propsFile);
            e.printStackTrace();
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {/*Ignoring*/ }
            }
        }
        
      
        props.setProperty("profiler", properties[0]);
        props.setProperty( "remote", properties[1]);
        props.setProperty("track.object.alloc", properties[2]);        
        props.setProperty("port", properties[3]);
        props.setProperty("thread-depth", properties[4]);
        props.setProperty("thread.compact.threshold.ms", properties[5] );
        props.setProperty("max-method-count", properties[6]);
        props.setProperty("method.compact.threshold.ms", properties[7] );
        
        FileOutputStream os = null;
        try {
            os = new FileOutputStream(propsFile);
            props.store(os, "no comments");
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (os != null) {
                try {
                    os.close();
                } catch (IOException e) {/*Ignoring*/}
            }
        }
    }
    
    private void setLocation() {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int x = (int) ((screenSize.getWidth() - getWidth()) / 2);
        int y = (screenSize.height - getHeight()) / 2;
        setLocation(new Point(x, y));
    }
    
    private void createGUI() {
        Container c = getContentPane();
        
        JPanel panel = new JPanel();
        
        JTable table1 = new JTable();
        table1.setTableHeader(null);
        table1.setRowSelectionAllowed(false);
        table1.setModel(new BooleanProps());
        table1.getColumnModel().getColumn(0).setMinWidth(200);
        table1.getColumnModel().getColumn(1).setMaxWidth(40);        
        
        JTable table2 = new JTable();
        table2.setTableHeader(null);
        table2.setRowSelectionAllowed(false);
        table2.setModel(new IntProps());
        table2.getColumnModel().getColumn(0).setMinWidth(200);
        table2.getColumnModel().getColumn(1).setMaxWidth(40);
        
        
        //panel.add(table1);
        panel.add(table2);
        
        panel.setBorder(new BevelBorder(0));
        c.add(panel);
        
        JPanel tools = new JPanel();
        tools.add(new JButton(saveAction));
        tools.add(new JButton(cancelAction));
                
        tools.setLayout(new FlowLayout());
        tools.setBorder(new BevelBorder(0));
        
        c.add(tools, SpringLayout.SOUTH);
        
        JPanel header = new JPanel();
        header.add(new JLabel("Profiling configuration"));
        header.setBorder(new BevelBorder(0));
        
        c.add(header, SpringLayout.NORTH);
        
        KeyStroke ks = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
        this.getRootPane().getInputMap().put(ks, "cancel");
        this.getRootPane().getActionMap().put("cancel", cancelAction);
        
        table2.getInputMap().put(ks, "cancel");
        table2.getActionMap().put("cancel", cancelAction);
        
    }
    



}
