package org.kozyrev.gui;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Point;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.filechooser.FileFilter;


public class ProfileWizard extends JDialog {
    
    public static final int ERROR_OPTION = -1;
    
    public static final int RUN_OPTION = 0;
    
    public static final int CANCEL_OPTION = 1;
        
    private static final JFileChooser jarOpener = new JFileChooser();
    /**
     */
    private static final long serialVersionUID = 4582027144615987988L;
    
    private int returnValue = ERROR_OPTION;
    
    private JTextField launchPath =  new JTextField(30);
    
    private JTextField programArguments = new JTextField(30);
    
    private JTextField VMArguments = new JTextField(30);
    
    private JTextArea excludedPaskages = new JTextArea(5, 30);
    
    private static final String DEFAULT_PROPERTY_FILE_NAME = "util/profile.properties";
    
    private final Action cancelAction = new CancelAction();
    
    private final Action runAction = new RunAction(); 

    private class ChangePathAction extends AbstractAction {
        /**
         * 
         */
        private static final long serialVersionUID = -3704017633047304255L;

        public ChangePathAction() {
            ClassLoader cl = this.getClass().getClassLoader();
            ImageIcon openIcon = new ImageIcon(cl
                    .getResource("images/document-open.png"));
            
            putValue(Action.SMALL_ICON, openIcon);
            putValue(Action.SHORT_DESCRIPTION, "Search");
        }
        
        public void actionPerformed(ActionEvent e) {
            if (jarOpener.showOpenDialog(ProfileWizard.this) 
                    == JFileChooser.APPROVE_OPTION) {
                
                launchPath.setText(jarOpener.getSelectedFile()
                        .getAbsolutePath());
            }
        }
    }
    
    private class RunAction extends AbstractAction {
        public RunAction() {
            putValue(Action.NAME, "Run");            
        }

        public void actionPerformed(ActionEvent e) {
            returnValue = RUN_OPTION;
            
            Properties props = new Properties();
            String propsFile = DEFAULT_PROPERTY_FILE_NAME;
            
            FileInputStream is = null;
            
            try {
                File f = new File(propsFile);
                if (!f.exists()) {
                    f.createNewFile();
                }
                is = new FileInputStream(propsFile);
                props.load(is);
            } catch (FileNotFoundException e1) {
                e1.printStackTrace();
            } catch (IOException e2) {
                System.err.print("Unable to open ");
                System.err.print(propsFile);
                e2.printStackTrace();
            } finally {
                if (is != null) {
                    try {
                        is.close();
                    } catch (IOException e2) {/*Ignoring*/}
                }
            }
            
            props.setProperty("exclude", excludedPaskages.getText());
            
            FileOutputStream os = null;
            try {
                os = new FileOutputStream(propsFile);
                props.store(os, "no comments");
            } catch (IOException e1) {
                e1.printStackTrace();
            } finally {
                if (os != null) {
                    try {
                        os.close();
                    } catch (IOException e1) {/*Ignoring*/}
                }
            }
            
            setVisible(false);  
        }

        private static final long serialVersionUID = -3127947817709670183L;
    }
    
    private class CancelAction extends AbstractAction {
        public CancelAction() {
            putValue(Action.NAME, "Cancel");
        }
        
        public void actionPerformed(ActionEvent e) {
            returnValue = CANCEL_OPTION;            
            setVisible(false);
        }
        
        private static final long serialVersionUID = -8929664901232147856L;
    }
    
    public ProfileWizard() {
        getContentPane().add(createGUI());
        pack();
        setModal(true);
        
        FileFilter jarFilter = new FileFilter() {

            @Override
            public boolean accept(File f) {
                return (f.isDirectory())
                        || f.getAbsolutePath().endsWith(".jar");
            }

            @Override
            public String getDescription() {
                return " java  [*.jar]";
            }

        };
        
        jarOpener.setApproveButtonText("");
        jarOpener.setFileSelectionMode(JFileChooser.FILES_ONLY);
        jarOpener.setFileFilter(jarFilter);

        addWindowListener(new WindowAdapter() {
            @Override
            public void windowClosing(WindowEvent e) {
                returnValue = CANCEL_OPTION;
            }
        });
        
        setLocation();
        
    }
    
    public int showRunDialog() {
        returnValue = ERROR_OPTION;
        setVisible(true);
        return returnValue;
    }
    
    public String getSelectedFile() {
        return launchPath.getText();
    }
    
    public String getVMArguments() {
        return VMArguments.getText();
    }
    
    public String getProgramArguments() {
        return programArguments.getText();
    }
    
    private void setLocation() {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        int x = (int) ((screenSize.getWidth() - getWidth()) / 2);
        int y = (screenSize.height - getHeight()) / 2;
        setLocation(new Point(x, y));
    }
    
    private JComponent createGUI() {
        JPanel mainPanel = new JPanel();
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));
        
        JPanel options = new JPanel();        
        options.setLayout(new BoxLayout(options, BoxLayout.Y_AXIS));
        options.setBorder(BorderFactory.createCompoundBorder(
                BorderFactory.createTitledBorder("Options"),
                BorderFactory.createEmptyBorder(5, 5, 5, 5)));
        
        JPanel path = new JPanel();
        path.setLayout(new BoxLayout(path, BoxLayout.X_AXIS));
        path.add(new JLabel("Launch path:"));
        path.add(Box.createHorizontalStrut(12));
        path.add(launchPath);
        path.add(new JButton(new ChangePathAction()) {
            private static final long serialVersionUID = -4606475281651081999L;

            {
                setPreferredSize(new Dimension(20, 20));
            }
        });
        
        JPanel progArgs = new JPanel();
        progArgs.setLayout(new BoxLayout(progArgs, BoxLayout.X_AXIS));
        progArgs.add(new JLabel("Program arguments:"));
        progArgs.add(Box.createHorizontalStrut(12));
        progArgs.add(programArguments);
        
        JPanel VMArgs = new JPanel();
        VMArgs.setLayout(new BoxLayout(VMArgs, BoxLayout.X_AXIS));
        VMArgs.add(new JLabel("VM arguments:"));
        VMArgs.add(Box.createHorizontalStrut(12));
        VMArgs.add(VMArguments);
        
        options.add(path);
        options.add(Box.createVerticalStrut(12));
        options.add(progArgs);
        options.add(Box.createVerticalStrut(12));
        options.add(VMArgs);
        
        JPanel packages = new JPanel();
        packages.setBorder(BorderFactory.createTitledBorder("Exclude packages(comma separated list):"));
        packages.setLayout(new BoxLayout(packages, BoxLayout.X_AXIS));
        packages.add(new JScrollPane(excludedPaskages));
        
        JPanel controls = new JPanel();
        controls.setLayout(new FlowLayout());
        controls.add(new JButton(runAction));
        JButton cancel = new JButton(cancelAction); 
        controls.add(cancel);        
        KeyStroke ks = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
        this.getRootPane().getInputMap().put(ks, "cancel");
        this.getRootPane().getActionMap().put("cancel", cancelAction);
        
        launchPath.getInputMap().put(ks, "cancel");
        launchPath.getActionMap().put("cancel", cancelAction);
        
        programArguments.getInputMap().put(ks, "cancel");
        programArguments.getActionMap().put("cancel", cancelAction);
        
        excludedPaskages.getInputMap().put(ks, "cancel");
        excludedPaskages.getActionMap().put("cancel", cancelAction);
        
        VMArguments.getInputMap().put(ks, "cancel");
        VMArguments.getActionMap().put("cancel", cancelAction);
        
        mainPanel.add(options);
        mainPanel.add(packages);
        mainPanel.add(controls);
        return mainPanel;
    }
    

}
