package org.kozyrev.gui.barchart;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.Iterator;

public abstract class BarChartModel {
    
    private ArrayList<ModelChangeListener> myListeners = new ArrayList<ModelChangeListener>();

    public abstract int getBarCount();
    
    public abstract String getBarText(int index);

    public abstract int getBarWeight(int index);

    public abstract Color getColor(int index);

    public abstract void addBar(String text, int weight, Color color);

    public abstract void removeBar(String text);
    
    public abstract void clear();
    
    abstract Bar getBar(int index);
    
    public abstract Comparator<Bar> getSorter();
    
    public abstract void setSorter(Comparator<Bar> sorter);
    
    public void addModelChangeListener(ModelChangeListener listener) {
        myListeners.add(listener);
    }
    
    public void removeModelChangeListener(ModelChangeListener listener) {
        
        for (int i = 0; i < myListeners.size(); i++) {
            if (listener.equals(myListeners.get(i))) {
                myListeners.remove(i);
                return;
            }
        }
    }
    
    protected void fireModelChanged() {
        resorting();
        
        for (Iterator<ModelChangeListener> it = myListeners.iterator(); it.hasNext();) {
            it.next().modelChanged();
        }
    }
    
    protected abstract void resorting();
    
    public static interface ModelChangeListener {
                
        public void modelChanged();
    }
    
    
    public static class Bar {
        
        private String text;
        private int weight;
        private Color color;
        
        public Bar (String text, int weight, Color color) {
            this.color = color;
            this.text = text;
            this.weight = weight;
        }

        public Color getColor() {
            return color;
        }

        public String getText() {
            return text;
        }

        public int getWeight() {
            return weight;
        }          
        
    }
}