package org.kozyrev.gui.barchart;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

public class DefaultBarChartModel extends BarChartModel {
    
    private ArrayList<Bar> myBars = new ArrayList<Bar>();
    private Comparator<Bar> comparator = new Comparator<Bar>() {
        public int compare(Bar o1, Bar o2) {
            Integer i1 = o1.getWeight();
            Integer i2 = o2.getWeight();
            return i2.compareTo(i1);
        }        
    };
    
    public DefaultBarChartModel() {
        
    }
    
    /* (non-Javadoc)
     * @see org.kozyrev.gui.barchart.BarChartModel#getBarCount()
     */
    public int getBarCount() {
        return myBars.size();
    }
    
    /* (non-Javadoc)
     * @see org.kozyrev.gui.barchart.BarChartModel#getBarText(int)
     */
    public String getBarText(int index) {
        return myBars.get(index).getText();
    }
    
    /* (non-Javadoc)
     * @see org.kozyrev.gui.barchart.BarChartModel#getBarWeight(int)
     */
    public int getBarWeight(int index) {
        return myBars.get(index).getWeight();
    }

    /* (non-Javadoc)
     * @see org.kozyrev.gui.barchart.BarChartModel#getColor(int)
     */
    public Color getColor(int index) {
        return myBars.get(index).getColor();
    }
    
    /* (non-Javadoc)
     * @see org.kozyrev.gui.barchart.BarChartModel#addBar(java.lang.String, int, java.awt.Color)
     */
    public void addBar(String text, int weight, Color color) {
        myBars.add(new Bar(text, weight, color));
        fireModelChanged();
    }
    
    /* (non-Javadoc)
     * @see org.kozyrev.gui.barchart.BarChartModel#removeBar(java.lang.String)
     */
    public void removeBar(String text) {
        for (int i = 0; i < myBars.size(); i++) {
            if (myBars.get(i).getText().equals(text)) {
                myBars.remove(i);
                return;
            }
        }
        
        fireModelChanged();
    }

    @Override
    public Comparator<Bar> getSorter() {
        return comparator;
    }

    @Override
    Bar getBar(int index) {
        return myBars.get(index);
    }

    @Override
    public void clear() {
        myBars.clear();
        fireModelChanged();        
    }
    
    @Override
    protected void resorting() {
        Collections.sort(myBars, getSorter());         
    }
    
    @Override
    public void setSorter(Comparator<Bar> sorter) {
        this.comparator = sorter;        
    }
}
