package org.kozyrev.gui.classchooser;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.ListCellRenderer;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;

public class ClassListCellRenderer implements ListCellRenderer {

    private final int WIDTH = 250;
    private final int HEIGHT = 40;
    
    JPanel renderer = new JPanel();
    JPanel textPanel;
    JCheckBox optionFlag = new JCheckBox();
    JLabel optionText = new JLabel();
    
    public ClassListCellRenderer() {
        renderer.setLayout(new BorderLayout());        
        renderer.setPreferredSize(new Dimension(WIDTH, HEIGHT));
        renderer.setBorder(new LineBorder(Color.BLACK, 1, true));
        renderer.add(optionFlag, BorderLayout.WEST);
        
        textPanel = new JPanel();
        textPanel.setBackground(Color.WHITE);
        textPanel.setLayout(new BorderLayout());
        textPanel.setBorder(new EmptyBorder(5,5,5,5));
        textPanel.add(optionText);
                    
        renderer.add(textPanel);   
        
        /*ClassLoader cl = this.getClass().getClassLoader();
        ImageIcon icon = new ImageIcon(cl
                .getResource("images/list-remove.png"));
        
        JLabel l = new JLabel();
        l.setIcon(icon);
        l.setBackground(Color.gray);
        l.setBorder(BorderFactory.createCompoundBorder(
                    BorderFactory.createLineBorder(Color.black), 
                    BorderFactory.createEmptyBorder(1, 1, 1, 1)));
        
        renderer.add(l, BorderLayout.EAST);*/
        
    }
    
    public Component getListCellRendererComponent(JList list, Object value,
            int index, boolean isSelected, boolean cellHasFocus) {
        
        if (isSelected) {
            textPanel.setBackground(renderer.getBackground());
        } else {
            textPanel.setBackground(Color.white);
        }
        
        if (value instanceof Option) {
            optionFlag.setSelected(((Option) value).isAllowed());
            optionText.setText(((Option) value).getText());
        } else {
            optionFlag.setSelected(false);
            optionText.setText("<empty>");
        }
        
        
        return renderer;
    }
    
    public static class ClassListKeyListener extends KeyAdapter {
        @Override
        public void keyPressed(KeyEvent e) {
            if (e.getKeyCode() == KeyEvent.VK_DELETE) {
                if (JOptionPane.showConfirmDialog(null, "Remove this rule?", "", JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION) {
                    ClassList packageList = ClassList.getInstance();
                    ClassListModel listModel = ClassListModel.getInstance();
                    
                    listModel.remove(packageList.getSelectedIndex());    
                }                
            }
            
            if (e.getKeyCode() == KeyEvent.VK_ENTER) {
                ClassList packageList = ClassList.getInstance();
                ClassListModel listModel = ClassListModel.getInstance();
                
                int selectedIndex = packageList.getSelectedIndex();
                Option selectedOption = 
                    (Option) listModel.elementAt(selectedIndex);
                
                String newRule = JOptionPane.showInputDialog("Edit rule", selectedOption.getText());
                
                if (newRule != null) {                
                    selectedOption.setText(newRule);
                    listModel.setElementAt(selectedOption, selectedIndex);
                    
                    packageList.repaint();
                }         
            }
        }
    }
    
    public static class PackageListElementListener extends MouseAdapter {
        @Override
        public void mouseClicked(MouseEvent e) {
            maybePerformClick(e);
        }
        
        private void maybePerformClick(MouseEvent e) {
            if (e.getButton() == MouseEvent.BUTTON1) {
                if (e.getClickCount() == 1) {
                    ClassList packageList = ClassList.getInstance();
                    ClassListModel listModel = ClassListModel.getInstance();
                    
                    int selectedIndex = packageList.locationToIndex(e.getPoint());
                    Option selectedOption = 
                        (Option) listModel.elementAt(selectedIndex);
                         
                    
                    selectedOption.setAllowed(!selectedOption.isAllowed());
                    listModel.setElementAt(selectedOption, selectedIndex);
                    
                    packageList.repaint();
                } else {
                    if (e.getClickCount() == 2) {     
                        
                        ClassList packageList = ClassList.getInstance();
                        ClassListModel listModel = ClassListModel.getInstance();
                        
                        int selectedIndex = packageList.locationToIndex(e.getPoint());
                        Option selectedOption = 
                            (Option) listModel.elementAt(selectedIndex);
                        
                        String newRule = JOptionPane.showInputDialog("Edit rule", selectedOption.getText());
                        
                        if (newRule != null) {                
                            selectedOption.setText(newRule);
                            listModel.setElementAt(selectedOption, selectedIndex);
                            
                            packageList.repaint();
                        }         
                    }
                }
            }
        }
    }

    
}
