package org.kozyrev.gui.util;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.regex.Pattern;

import org.apache.log4j.Logger;
import org.kozyrev.gui.Main;

public class ClassFilter {

    private Politic myPolitic;
    private Set<Pattern> myFormattedExpr = new HashSet<Pattern>();
    private static final Logger LOG = Logger.getLogger(Main.class);
    private static ClassFilter myInstance = new ClassFilter();
    private ArrayList<FilterChangeListener> myChangeListeners = new ArrayList<FilterChangeListener>();
    
    private ClassFilter() {
        setPolitic(Politic.NO_ACTION);
    }
    
    public static ClassFilter getInstance() {
        return myInstance;
    }
    
    public void setPolitic(Politic politic) {
        myPolitic = politic;
        fireFilterChanged();
    }
    
    
    public Politic getPolitic() {
        return myPolitic;
    }
    
    public void addPackageRegExp(String s) {
        myFormattedExpr.add(Pattern.compile(formatExpression(s)));
        fireFilterChanged();
    }
    
    public void clear() {
        myFormattedExpr.clear();
        fireFilterChanged();
    }
    
    public boolean accept(String test) {
        
        if (myPolitic == Politic.NO_ACTION) {
            return true;
        }
        
        boolean accepted = false;
        if (myPolitic == Politic.DECLINE) {
            accepted = true;
        }
        
                
        for (Iterator<Pattern> it = myFormattedExpr.iterator(); it.hasNext();) {            
            Pattern expr = it.next();
            
            if (expr.matcher(test).matches()) {
                if (myPolitic == Politic.ACCEPT) {
                    accepted = true;
                } else {
                    accepted = false;
                }
            }
        }
        
        return accepted;
    }
    
    public void addFilterChangeListener(FilterChangeListener l) {
        myChangeListeners.add(l);
    }
    
    public void removeFilterChangeListener(FilterChangeListener l) {
        myChangeListeners.remove(l);   
    }
    
    protected void fireFilterChanged() {
        for (FilterChangeListener l : myChangeListeners) {
            l.filterChanded(this);
        }
    }
    
    private String formatExpression(String unformatted) {
        String result = unformatted.replaceAll("\\.", "\t");
        result = result.replaceAll("\t", "\\\\.");
        
        result = result.replaceAll("\\*", "\t");
        result = result.replaceAll("\t", "[\\\\w\\\\W\\\\d]*");
        
        LOG.debug("result = " + result);
        
        return result;
    }
    
    public enum Politic {
        ACCEPT, DECLINE, NO_ACTION;
    }
    
    public interface FilterChangeListener {
        public void filterChanded(ClassFilter source);
    }
}
