package org.kozyrev.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class FrameRecord {
    private int myCount;
    private long myTime;
    private ArrayList<FrameRecord> myFrames = new ArrayList<FrameRecord>();
    private MethodRecord myMethod;
    
    public FrameRecord(MethodRecord method, int count, long time) {
        myMethod = method;
        myCount = count;
        myTime = time;
        
        myMethod.addFrame(this);
    }
    
    public FrameRecord(MethodRecord method, int count, long time, 
            FrameRecord child) {
        
        this(method, count, time); 
        myFrames.add(child);
    }
    
    public int getCallCount() {
        return myCount;
    }
    
    public long getTime() {
        return myTime;
    }
    
    public MethodRecord getMethod() {
        return myMethod;
    }
    
    public List<FrameRecord> getCalledFrames() {
        return Collections.unmodifiableList(myFrames);
    }
    
    public void addFrame(FrameRecord frame) {
        myFrames.add(frame);
    }
    
    public boolean hasChildren() {
        return !myFrames.isEmpty();
    }
    
}
