package org.kozyrev.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class MethodRecord implements Comparable<MethodRecord>{
    
    private String myName;
    private ClassRecord myClass;
    private ArrayList<FrameRecord> myFrames = new ArrayList<FrameRecord>();
    
    public MethodRecord(String name, ClassRecord classRecord) {
        myName = name;
        myClass = classRecord;
        
        if (myName.indexOf("<init>") >= 0) {
            myClass.addConstructor(this);
        } else {
            myClass.addMethod(this);
        }
        
    }
    
    public ClassRecord getDeclaringClass() {
        return myClass;
    }
    
    public String getName() {
        return myName;    
    }
    
    public String getShortName() {
        return myName.substring(myName.indexOf(":") + 1);
    }
    
    public List<FrameRecord> getFrames() {
        return Collections.unmodifiableList(myFrames);   
    }
    
    public void addFrame(FrameRecord record) {
        myFrames.add(record);
    }
    
    public long getTotalTime() {
        
        long result = 0;
        for (FrameRecord f : myFrames) {
            result += f.getTime();
        }
        
        return result;
    }

    public int compareTo(MethodRecord o) {
        String name1 = this.getName();
        String name2 = o.getName();
        
        if (name1.endsWith("<init>")) {
            name1 = this.getDeclaringClass().getName() 
                    + ":" + this.getDeclaringClass().getShortName();;
        }
        
        if (name2.endsWith("<init>")) {
            name2 = o.getDeclaringClass().getName()  
                    + ":" + this.getDeclaringClass().getShortName();;
        }
        
        
        return name1.compareTo(name2);
    }
    
    @Override
    public boolean equals(Object obj) {
               
        if (obj instanceof MethodRecord) {
            String name1 = this.getName();
            String name2 = ((MethodRecord) obj).getName();
            
            if (name1.endsWith("<init>")) {
                name1 = this.getDeclaringClass().getName()
                        + ":" + this.getDeclaringClass().getShortName();
            }
            
            if (name2.endsWith("<init>")) {
                name2 = ((MethodRecord) obj).getDeclaringClass().getName() 
                        + ":" + this.getDeclaringClass().getShortName();;
            }
            
            return name1.equals(name2);
        } else {
            return false;
        }
    }
    
    @Override
    public int hashCode() {
        String name1 = this.getName();
                
        if (name1.endsWith("<init>")) {
            name1 = this.getDeclaringClass().getName()  + ":" 
                    + this.getDeclaringClass().getShortName();;
            
        }
                
        return name1.hashCode();
    }
    
    
}
