package org.kozyrev.model.xml;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Stack;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.log4j.Logger;
import org.kozyrev.model.ClassRecord;
import org.kozyrev.model.FrameRecord;
import org.kozyrev.model.MethodRecord;
import org.kozyrev.model.ProfileModel;
import org.kozyrev.model.ThreadRecord;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

public class XMLParser {

    private static final SAXParserFactory PARSER_FACTORY;

    private static SAXParser parser = null;

    private static final Logger LOG = Logger.getLogger(XMLParser.class);

    private static final Stack STACK = new Stack();

    private static final ArrayList<ThreadRecord> THREAD_LIST = new ArrayList<ThreadRecord>();

    private static final HashMap<String, ClassRecord> CLASS_MAP = new HashMap<String, ClassRecord>();
    
    private static final HashMap<String, MethodRecord> METHOD_MAP = new HashMap<String, MethodRecord>();
    
    static {
        PARSER_FACTORY = SAXParserFactory.newInstance();

        try {
            parser = PARSER_FACTORY.newSAXParser();
        } catch (ParserConfigurationException e) {
            LOG.error("   XMLParser", e);
        } catch (SAXException e) {
            LOG.error("   XMLParser", e);
        }
    }

    private XMLParser() {

    }

    public static ProfileModel parseFile(File file)
            throws ParserConfigurationException, SAXException, IOException {

        if (parser == null) {
            LOG.error("XMLParser    ");
            throw new ParserConfigurationException(
                    "XMLParser    ");
        }

        init();

        parser.parse(file, new CallGraphParseHandler());

        dispose();

        return null;
    }

    private static void dispose() {
    
    }

    private static void init() {
        STACK.clear();
        THREAD_LIST.clear();
        CLASS_MAP.clear();
    }



    private static void checkXMLSyntax(String element, ElementAlias alias)
            throws SAXException {
    
        if (!element.equals(alias)) {
            String message = "Unexpected element closing. \"" + alias
                    + "\" expected but was \"" + element
                    + "\"";
    
            throw new SAXException(message);
        }
    
        if (STACK.isEmpty()) {
            String message = "Unexpected element closing. \"" + alias
                    + "\" expected but was nothing";
    
            throw new SAXException(message);
        }
    }
    
    private static class CallGraphParseHandler extends DefaultHandler {

        @Override
        public void startElement(String uri, String localName, String qName,
                Attributes attributes) throws SAXException {
                        
        }

        @Override
        public void endElement(String uri, String localName, String qName)
                throws SAXException {

            checkXMLSyntax(localName, ElementAlias.FRAME);
            
        }

        private void processFrameEnd() throws SAXException {
                        
        }

        private void processThreadEnd() {
            
        }
        
        private void processInteractionEnd() {
        }
        
    }

    private static class DocumentElement {

        private String myLocalName;

        private Attributes myAttributes;

        public DocumentElement(String localName, Attributes attributes) {
            myAttributes = attributes;
            myLocalName = localName;
        }

        public Attributes getAttributes() {
            return myAttributes;
        }

        public String getLocalName() {
            return myLocalName;
        }
    }

    private enum ElementAlias {

        PROFILE("profile"), THREAD("thread"), INTERACTION("interaction"), FRAME(
                "frame"), CLASS_MAP("class-map"), ENTRY("entry"), ALLOCATION(
                "allocation"), CLASS("class");

        private String myAlias;

        private ElementAlias(String alias) {
            myAlias = alias;
        }

        @Override
        public String toString() {
            return myAlias;
        }
    }

    private enum AttributeAlias {

        CLASS_NAME("cn"), METHOD_NAME("mn"), COUNT("c"), SHORT_NAME("s"), FULL_NAME(
                "f"), ID("id"), NANOTIME("t");

        private String myAlias;

        private AttributeAlias(String alias) {
            myAlias = alias;
        }

        @Override
        public String toString() {
            return myAlias;
        }
    }

}
