package org.kozyrev.net;

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.UnknownHostException;

import org.apache.log4j.Logger;

public class ConnectionController {
    
    private static Socket mySocket = null;
    private static final String myURL = "localhost";
    private static int myPort = 15599;
    
    private static final int TIMEOUT = 1500; 
    private static final Logger LOG = Logger.getLogger(ConnectionController.class);
    
    private ConnectionController() {        
    }
    
    public static Socket getSocket(int port) throws IOException {
        
        myPort = port;
        int totalTime = 0;
        
        try {
            if (mySocket != null) {
                mySocket.close();    
            }            
        } catch (IOException e1) {
            LOG.error("Can't connect to profiler", e1);
            throw e1;
        }
        
        mySocket = null;
        
        while (mySocket == null) {
            try {
                mySocket = new Socket(myURL, myPort);
            } catch (UnknownHostException e) {/*Ignoring*/} 
              catch (IOException e) {/*Ignoring*/}
              
            if (mySocket == null) {
                
                if (totalTime >= TIMEOUT) {
                    LOG.error("Can't connect to profiler");
                    throw new UnknownHostException();                    
                }
                
                try {
                    totalTime += 100;
                    Thread.sleep(100);
                } catch (InterruptedException e) {/*Ignoring*/}
            }
        }
        
        return mySocket;
   }
    
    public static int getPortNumber() {
        return myPort;
    }
    
    public static boolean isPortFree(int port) {
        ServerSocket serverSocket;
        try {
            serverSocket = new ServerSocket(port);
            serverSocket.close();
            return true;
        } catch (IOException e) {
            return false;
        }
    }
    
    public static int getFreePort(int basePort) {
        for (int i = basePort; i < 65536; i++) {
            if (isPortFree(i)) {
                return i;
            }
        }
        
        throw new RuntimeException("Free ports not available");
    }
}
