package org.kozyrev.tests;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;

import junit.framework.Assert;

import org.junit.Test;
import org.kozyrev.gui.util.GlobalState;
import org.kozyrev.gui.util.GlobalStateManager;


public class GlobalStateTest {

    @Test
    public void testState() {
        GlobalState state = new GlobalState();
        
        Action[] actions = new Action[10];
        
        for (int i = 0; i < actions.length; i++) {
            actions[i] = new TestAction();
            state.addAction(actions[i]);
        }
        
        state.enable();
        Assert.assertTrue(isAllSame(actions, true));
        
        state.disable();
        Assert.assertTrue(isAllSame(actions, false));
        
        state.enable();
        Assert.assertTrue(isAllSame(actions, true));
        
        state.disable();
        Assert.assertTrue(isAllSame(actions, false));
    }
    
    @Test
    public void testManager() {
        GlobalStateManager manager = new GlobalStateManager();
        
        //Creating of 2 states
        GlobalState stateOne = new GlobalState();
        GlobalState stateTwo = new GlobalState();        
        
        Action[] actionsOne = new Action[10];
        
        for (int i = 0; i < actionsOne.length; i++) {
            actionsOne[i] = new TestAction();
            stateOne.addAction(actionsOne[i]);
        }
        
        Action[] actionsTwo = new Action[10];
        
        for (int i = 0; i < actionsTwo.length; i++) {
            actionsTwo[i] = new TestAction();
            stateTwo.addAction(actionsTwo[i]);
        }
        
        manager.addState(stateOne);
        manager.addState(stateTwo);
        
        manager.setState(stateOne);        
        Assert.assertTrue(isAllSame(actionsOne, true));
        Assert.assertTrue(isAllSame(actionsTwo, false));
        
        manager.setState(stateTwo);        
        Assert.assertTrue(isAllSame(actionsOne, false));
        Assert.assertTrue(isAllSame(actionsTwo, true));
        
        manager.setState(stateOne);        
        Assert.assertTrue(isAllSame(actionsOne, true));
        Assert.assertTrue(isAllSame(actionsTwo, false));
        
        for (int i = 0; i < actionsOne.length; i++) {
            stateTwo.addAction(actionsOne[i]);
        }
        
        manager.addState(stateOne);
        Assert.assertTrue(isAllSame(actionsOne, true));
        
    }
    
    /**
     * Tests all actions in array is enabled or disabled
     * 
     * @param actions
     * @param test
     * @return
     */
    private boolean isAllSame(Action[] actions, boolean test) {
                
        for (int i = 0; i < actions.length; i++) {
            if (actions[i].isEnabled() != test) {
                return false;
            }
        }
        
        return true;
    }
    
    
    @SuppressWarnings("serial")
    private class TestAction extends AbstractAction {

        public void actionPerformed(ActionEvent e) {
            
        }
    }

}
