package org.kozyrev.tests;

import org.junit.Assert;
import org.junit.Test;
import org.kozyrev.model.ClassRecord;
import org.kozyrev.model.FrameRecord;
import org.kozyrev.model.MethodRecord;
import org.kozyrev.model.ThreadRecord;


public class TestModel {

    @Test
    public void testClassRecord() {
        ClassRecord record = new ClassRecord("long", "short", 0);
        
        Assert.assertEquals("long", record.getName());
        Assert.assertEquals("short", record.getShortName());
        Assert.assertEquals(0, record.getAllocationCount());
        
        Assert.assertEquals(0, record.getConstructors().size());
        Assert.assertEquals(0, record.getMethods().size());
    }
    
    @Test
    public void testMethodRegistering() {
        ClassRecord record = new ClassRecord("long", "short", 0);
        
        MethodRecord constructor;
        MethodRecord method;
        
        record.addConstructor(constructor = new MethodRecord("init",  
                new ClassRecord("long", "short", 0)));
        
        record.addMethod(method = new MethodRecord("doIt",  
                new ClassRecord("long", "short", 0)));
        
        Assert.assertEquals(constructor, record.getConstructors().get(0));
        Assert.assertEquals(1, record.getConstructors().size());
        
        Assert.assertEquals(method, record.getMethods().get(0));
        Assert.assertEquals(1, record.getMethods().size());
    }
    
    @Test
    public void testFrameRecord() {
        FrameRecord record = new FrameRecord(new MethodRecord("myMethod", 
                new ClassRecord("long", "short", 777)), 777, 777);
        
        Assert.assertEquals(777, record.getCallCount());
        Assert.assertEquals((long)777, record.getTime());
        Assert.assertEquals(0, record.getCalledFrames().size());
        
        if (record.getMethod() == null) {
            Assert.fail("Method has unreachable from accessor method");
        }
    }
    
    @Test
    public void testMethodAdding() {
        FrameRecord record = new FrameRecord(new MethodRecord("myMethod", 
                new ClassRecord("long", "short", 777)), 777, 777);
        
        FrameRecord peek;
        
        record.addFrame(peek = new FrameRecord(new MethodRecord("myMethod", 
                new ClassRecord("long", "short", 1)), 1, 1));
        
        record.addFrame(new FrameRecord(new MethodRecord("myMethod", 
                new ClassRecord("long", "short", 2)), 2, 2));
        
        Assert.assertEquals(2, record.getCalledFrames().size());
        Assert.assertEquals(peek, record.getCalledFrames().get(0));
    }
    
    
    @Test
    public void testMethodRecord() {
        ClassRecord cr;
        MethodRecord method = new MethodRecord("name", cr = new ClassRecord("long", "short", 0));
        
        Assert.assertEquals("name", method.getName());
        Assert.assertEquals(cr, method.getDeclaringClass());
    }
    
    @Test
    public void testFrameRegistering() {
        FrameRecord record = new FrameRecord(new MethodRecord("myMethod", 
                new ClassRecord("long", "short", 777)), 777, 777);
        
        MethodRecord method = new MethodRecord("name",  new ClassRecord("long", "short", 0));
        
        method.addFrame(record);
        method.addFrame(new FrameRecord(new MethodRecord("myMethod", 
                new ClassRecord("long", "short", 0)), 0, 0));
        
        Assert.assertEquals(record, method.getFrames().get(0));
        Assert.assertEquals(2, method.getFrames().size());
    }
    
    @Test
    public void testThreadRecord() {
        ThreadRecord record = new ThreadRecord("10");
        
        Assert.assertEquals("10", record.getID());
        Assert.assertEquals(0, record.getFrames().size());        
    }
    
    @Test
    public void testThreadAdding() {
        
        ThreadRecord record = new ThreadRecord("10");
        
        
        record.addFrame(new FrameRecord(new MethodRecord("myMethod", 
                new ClassRecord("long", "short",10)), 0, 0));
    }
}
